==> hermes-2.5.1.75.tar.gz: hermes-2.5.1.75/spec/apis/dmo/housing_channel_settings_spec.rb <==
require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    department: 'String',
    insee_code: 'String',
    housing_channel_keys: 'Array'
  }

  ACCEPTABLE_PARAMS = %w[
    department
    insee_code
    housing_channel_keys
    created_at
    updated_at
  ]

  RSpec.shared_examples 'admin only' do
    before do
      authenticated_header(request, user)
      subject
    end
    it 'returns unauthorized error' do
      expect(response).to have_http_status 401
    end
  end

  describe Api::Dmo::HousingChannelSettingsController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:roles) { %w[reservataire bailleur entreprise demandeur Manager Supervisor] }
    let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }
    let!(:housing_channel) { create :housing_channel, key: 'dalo' }

    let(:channel_setting) { create :housing_channel_setting }
    let(:params) do
      { attributes: { department: '83', housing_channel_keys: ['dalo'] } }
    end

    before { authenticated_header(request, responsible) }

    describe 'GET index' do
      let!(:channels) { create_list :housing_channel_setting, 5 }
      subject { get 'index' }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'GET show' do
      subject { get 'show', { id: channel_setting.id } }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq channel_setting.id.to_s
        expect(json['data']['attributes']['department']).to eq '34'
      end
    end

    describe 'POST create' do
      subject { post 'create', { data: params } }

      it_behaves_like 'admin only'

      it 'returns not found channel error' do
        data = { attributes: { department: '83', housing_channel_keys: ['unknown'] } }
        post 'create', { data: data }
        expect(response).to have_http_status 422
        expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
      end

      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end

      it 'creates a housing channel setting setting' do
        expect do
          post 'create', { data: params }
        end.to change { Hermes::HousingChannelSetting.count }.by 1
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['attributes']['department']).to eq '83'
        expect(json['data']['attributes']['housing_channel_keys']).to eq ['dalo']
      end
    end

    describe 'PATCH update' do
      subject { patch 'update', { id: channel_setting.id, data: params } }

      it_behaves_like 'admin only'

      it 'returns not found channel error' do
        data = { attributes: { department: '83', housing_channel_keys: ['unknown'] } }
        patch 'update', {  id: channel_setting.id, data: data }
        expect(response).to have_http_status 422
        expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
      end

      it 'succeeds' do
        subject
        expect(response).to have_http_status 200
      end

      it 'updates the channel setting' do
        expect do
          subject
        end.to change { channel_setting.reload.department }.from('34').to '83'
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq channel_setting.id.to_s
        expect(json['data']['attributes']['department']).to eq '83'
        expect(json['data']['attributes']['housing_channel_keys']).to eq ['dalo']
      end
    end

    describe 'DELETE destroy' do
      let(:rand_admin) { create :user, roles: ['Administrator'] }
      let!(:owned_channel_setting) { create :housing_channel_setting, user_id: responsible.id }

      subject { delete 'destroy', { id: owned_channel_setting.id.to_s } }

      it_behaves_like 'admin only'

      context 'for owner' do
        it 'succeeds' do
          subject
          expect(response).to have_http_status 204
        end

        it 'destroys the channel setting' do
          expect do
            subject
          end.to change { Hermes::HousingChannelSetting.count }.by -1
        end
      end

      context 'for another admin' do
        before { authenticated_header(request, rand_admin) }

        it 'returns error' do
          subject
          expect(response).to have_http_status 404
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::HousingChannelSettingsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS = 'API_DMO_HOUSING_CHANNEL_SETTINGS'

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) { create(:user, roles: ['Administrator']) }

    context 'as admin' do
      let!(:admin) { create(:administrator) }

      path '/api/dmo/housing_channel_settings' do
        post 'Create housing channel setting' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:department] = { type: :string, required: true }
          json_params[:insee_code] = { type: :string }
          json_params[:housing_channel_keys] = { type: :array }

          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'housing channel setting created' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let(:params) do
              jsonapi_attributes(attributes_for(:housing_channel_setting))
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_channel_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings' do
        get 'Retrieves all housing channel settings' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'

          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          security [Bearer: {}]
          paginator

          response '200', 'housing channel settings found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:housing_channel_settings) { create_list(:housing_channel_setting, 5) }

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { housing_channel_settings }
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings/{id}' do
        get 'Retrieves a single housing channel setting by id' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'housing channel setting found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:housing_channel_setting) { create(:housing_channel_setting) }
            let(:id) { housing_channel_setting.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { housing_channel_setting }
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings/{id}' do
        put 'edit housing channel setting' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'housing channel setting updated' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:housing_channel_setting) { create(:housing_channel_setting) }
            let(:id) { housing_channel_setting.id }
            let(:params) do
              jsonapi_attributes(
                 { department: 'string' }
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_channel_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings/{id}' do
        delete 'Delete a single housing channel setting by id' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'housing channel setting deleted' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:id) { create(:housing_channel_setting, responsible: admin.id).id }
            run_test!
          end
        end
      end
    end
  end
end

==> hermes-4.1.0.5.tar.gz: hermes-4.1.0.5/spec/apis/dmo/housing_channel_settings_spec.rb <==
require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    department: 'String',
    insee_code: 'String',
    housing_channel_keys: 'Array'
  }

  ACCEPTABLE_PARAMS = %w[
    department
    insee_code
    housing_channel_keys
    created_at
    updated_at
  ]

  RSpec.shared_examples 'admin only' do
    before do
      authenticated_header(request, user)
      subject
    end
    it 'returns unauthorized error' do
      expect(response).to have_http_status 401
    end
  end

  describe Api::Dmo::HousingChannelSettingsController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:roles) { %w[bailleur entreprise demandeur Manager Supervisor] }
    let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }
    let!(:housing_channel) { create :housing_channel, key: 'dalo' }

    let(:channel_setting) { create :housing_channel_setting }
    let(:params) do
      { attributes: { department: '83', housing_channel_keys: ['dalo'] } }
    end

    before { authenticated_header(request, responsible) }

    describe 'GET index' do
      let!(:channels) { create_list :housing_channel_setting, 5 }
      subject { get 'index' }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'GET show' do
      subject { get 'show', { id: channel_setting.id } }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq channel_setting.id.to_s
        expect(json['data']['attributes']['department']).to eq '34'
      end
    end

    describe 'POST create' do
      subject { post 'create', { data: params } }

      it_behaves_like 'admin only'

      it 'returns not found channel error' do
        data = { attributes: { department: '83', housing_channel_keys: ['unknown'] } }
        post 'create', { data: data }
        expect(response).to have_http_status 422
        expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
      end

      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end

      it 'creates a housing channel setting setting' do
        expect do
          post 'create', { data: params }
        end.to change { Hermes::HousingChannelSetting.count }.by 1
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['attributes']['department']).to eq '83'
        expect(json['data']['attributes']['housing_channel_keys']).to eq ['dalo']
      end
    end

    describe 'PATCH update' do
      subject { patch 'update', { id: channel_setting.id, data: params } }

      it_behaves_like 'admin only'

      it 'returns not found channel error' do
        data = { attributes: { department: '83', housing_channel_keys: ['unknown'] } }
        patch 'update', {  id: channel_setting.id, data: data }
        expect(response).to have_http_status 422
        expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
      end

      it 'succeeds' do
        subject
        expect(response).to have_http_status 200
      end

      it 'updates the channel setting' do
        expect do
          subject
        end.to change { channel_setting.reload.department }.from('34').to '83'
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq channel_setting.id.to_s
        expect(json['data']['attributes']['department']).to eq '83'
        expect(json['data']['attributes']['housing_channel_keys']).to eq ['dalo']
      end
    end

    describe 'DELETE destroy' do
      let(:rand_admin) { create :user, roles: ['Administrator'] }
      let!(:owned_channel_setting) { create :housing_channel_setting, user_id: responsible.id }

      subject { delete 'destroy', { id: owned_channel_setting.id.to_s } }

      it_behaves_like 'admin only'

      context 'for owner' do
        it 'succeeds' do
          subject
          expect(response).to have_http_status 204
        end

        it 'destroys the channel setting' do
          expect do
            subject
          end.to change { Hermes::HousingChannelSetting.count }.by -1
        end
      end

      context 'for another admin' do
        before { authenticated_header(request, rand_admin) }

        it 'returns error' do
          subject
          expect(response).to have_http_status 404
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::HousingChannelSettingsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS = 'API_DMO_HOUSING_CHANNEL_SETTINGS'

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) { create(:user, roles: ['Administrator']) }

    context 'as admin' do
      let!(:admin) { create(:administrator) }

      path '/api/dmo/housing_channel_settings' do
        post 'Create housing channel setting' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:department] = { type: :string, required: true }
          json_params[:insee_code] = { type: :string }
          json_params[:housing_channel_keys] = { type: :array }

          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'housing channel setting created' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let(:params) do
              jsonapi_attributes(attributes_for(:housing_channel_setting))
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_channel_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings' do
        get 'Retrieves all housing channel settings' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'

          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          security [Bearer: {}]
          paginator

          response '200', 'housing channel settings found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:housing_channel_settings) { create_list(:housing_channel_setting, 5) }

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { housing_channel_settings }
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings/{id}' do
        get 'Retrieves a single housing channel setting by id' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'housing channel setting found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:housing_channel_setting) { create(:housing_channel_setting) }
            let(:id) { housing_channel_setting.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { housing_channel_setting }
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings/{id}' do
        put 'edit housing channel setting' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'housing channel setting updated' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:housing_channel_setting) { create(:housing_channel_setting) }
            let(:id) { housing_channel_setting.id }
            let(:params) do
              jsonapi_attributes(
                 { department: 'string' }
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_channel_settings'
            end
          end
        end
      end

      path '/api/dmo/housing_channel_settings/{id}' do
        delete 'Delete a single housing channel setting by id' do
          tags TAGS_API_DMO_HOUSING_CHANNEL_SETTINGS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'housing channel setting deleted' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:id) { create(:housing_channel_setting, responsible: admin.id).id }
            run_test!
          end
        end
      end
    end
  end
end
